@@ -19,6 +19,7 @@ try:
 except ImportError:
     from .compat import cached_property
 
+from .actions import help_action
 from .exceptions import CommandDefinitionError, ParameterDefinitionError
 from .formatting.commands import CommandHelpFormatter
 from .parameters.base import ParamBase, Parameter, BaseOption, BasePositional
@@ -47,21 +48,20 @@ class CommandParameters:
     positionals: List[BasePositional]
     option_map: Dict[str, BaseOption]
 
-    def __init__(self, command: CommandType, command_parent: CommandType = None):
+    def __init__(self, command: CommandType, command_parent: Optional[CommandType], config: CommandConfig):
         self.command = command
-        if command_parent:
+        if command_parent is not None:
             self.command_parent = command_parent
             self.parent = command_parent.__class__.params(command_parent)
 
+        self.config = config
         self._process_parameters()
 
     def __repr__(self) -> str:
         positionals = len(self.positionals)
         options = len(self.options)
-        return (
-            f'<{self.__class__.__name__}[command={self.command.__name__},'
-            f' positionals={positionals!r}, options={options!r}]>'
-        )
+        cls_name = self.__class__.__name__
+        return f'<{cls_name}[command={self.command.__name__}, positionals={positionals!r}, options={options!r}]>'
 
     @property
     def pass_thru(self) -> Optional[PassThru]:
@@ -81,19 +81,21 @@ class CommandParameters:
 
     @cached_property
     def formatter(self) -> CommandHelpFormatter:
-        command = self.command
-        config: CommandConfig = command.__class__.config(command)
-        if config is None:
+        if self.config is None:
             formatter_factory = CommandHelpFormatter
         else:
-            formatter_factory = config.command_formatter or CommandHelpFormatter
-        formatter = formatter_factory(command, self)
+            formatter_factory = self.config.command_formatter or CommandHelpFormatter
+        formatter = formatter_factory(self.command, self)
         formatter.maybe_add_option(self._pass_thru)
         formatter.maybe_add_positionals(self.positionals)
         formatter.maybe_add_options(self.options)
         formatter.maybe_add_groups(self.groups)
         return formatter
 
+    @property
+    def _has_help(self) -> bool:
+        return help_action in self.always_available_action_flags or (self.parent and self.parent._has_help)
+
     # region Initialization
 
     def _process_parameters(self):
@@ -141,6 +143,9 @@ class CommandParameters:
             if param.group:
                 groups.update(_get_groups(param))
 
+        if self.config and self.config.add_help and (not self.parent or not self.parent._has_help):
+            options.append(help_action)
+
         self._process_positionals(positionals)
         self._process_options(options)
         self._process_groups(groups)
@@ -176,6 +181,8 @@ class CommandParameters:
                     self.sub_command = param
                 else:
                     self.action = param
+                    if not param.choices:
+                        raise CommandDefinitionError(f'No choices were registered for {self.action}')
 
             if param.nargs.variable and not param.choices:
                 var_nargs_param = param
@@ -227,7 +234,7 @@ class CommandParameters:
                 if len(params) > 1:
                     group = next((p.group for p in params if p.group), None)
                     if group and group.mutually_exclusive:
-                        if not all(p.group == group for p in params):
+                        if any(p.group != group for p in params):
                             invalid[(before_main, prio)] = params
                     else:
                         invalid[(before_main, prio)] = params
@@ -249,7 +256,7 @@ class CommandParameters:
             return None
         elif option.startswith('--'):
             try:
-                param, value = self.long_option_to_param_value_pair(option)
+                opt, param, value = self.long_option_to_param_value_pair(option)
             except KeyError:
                 return None
             else:
@@ -260,21 +267,21 @@ class CommandParameters:
             except KeyError:
                 return None
             else:
-                return tuple(param for param, value in param_value_pairs)
+                return tuple(param for opt, param, value in param_value_pairs)
         else:
             return None
 
-    def long_option_to_param_value_pair(self, option: str) -> Tuple[BaseOption, Optional[str]]:
+    def long_option_to_param_value_pair(self, option: str) -> Tuple[str, BaseOption, Optional[str]]:
         try:
-            return self.option_map[option], None
+            return option, self.option_map[option], None
         except KeyError:
             if '=' in option:
                 option, value = option.split('=', 1)
-                return self.option_map[option], value
+                return option, self.option_map[option], value
             else:
                 raise
 
-    def short_option_to_param_value_pairs(self, option: str) -> List[Tuple[BaseOption, Optional[str]]]:
+    def short_option_to_param_value_pairs(self, option: str) -> List[Tuple[str, BaseOption, Optional[str]]]:
         try:
             option, value = option.split('=', 1)
         except ValueError:
@@ -286,24 +293,24 @@ class CommandParameters:
             if value is not None:
                 raise
         else:
-            return [(param, value)]
+            return [(option, param, value)]
 
         key, value = option[1], option[2:]
         # value will never be empty if key is a valid option because by this point, option is not a short option
         param = self.combo_option_map[key]
         if param.would_accept(value, short_combo=True):
-            return [(param, value)]
+            return [(key, param, value)]
         else:
             combo_option_map = self.combo_option_map
-            return [(combo_option_map[c], None) for c in option[1:]]
+            return [(c, combo_option_map[c], None) for c in option[1:]]
 
     def find_option_that_accepts_values(self, option: str) -> Optional[BaseOption]:
         if option.startswith('--'):
-            param = self.long_option_to_param_value_pair(option)[0]
+            param = self.long_option_to_param_value_pair(option)[1]
             if param.accepts_values:
                 return param
         elif option.startswith('-'):
-            for param, _ in self.short_option_to_param_value_pairs(option):
+            for _, param, _ in self.short_option_to_param_value_pairs(option):
                 if param.accepts_values:
                     return param
         else:
