@@ -23,8 +23,6 @@ __all__ = ['CommandConfig', 'ShowDefaults', 'OptionNameMode']
 _ConfigValue = TypeVar('_ConfigValue')
 ConfigValue = Union[_ConfigValue, Any]
 
-# TODO: Config / handling for subcommand option conflict, especially for short form where name/long don't match previous
-
 
 # region Config Option Enums
 
@@ -75,17 +73,37 @@ class OptionNameMode(FixedFlag):
     :BOTH: Both ``--foo-bar`` and ``--foo_bar`` will be accepted
 
     If a long form is provided explicitly for a given optional Parameter, then this setting will be ignored.
+
+    The value may be specified to Commands as ``option_name_mode=<mode>`` or to Parameters as ``name_mode=<mode>``,
+    where ``<mode>`` is one of:
+
+        - ``OptionNameMode.UNDERSCORE`` or ``OptionNameMode.DASH`` or ``OptionNameMode.BOTH``
+        - ``'underscore'`` or ``'dash'`` or ``'both'``
+        - ``'_'`` or ``'-'`` or ``'*'``
     """
 
     UNDERSCORE = 1
     DASH = 2
     BOTH = 3
 
+    @classmethod
+    def _missing_(cls, value: Union[str, int]) -> OptionNameMode:
+        try:
+            return OPT_NAME_MODE_ALIASES[value]
+        except KeyError:
+            pass
+        return super()._missing_(value)
+
+
+OPT_NAME_MODE_ALIASES = {'-': OptionNameMode.DASH, '_': OptionNameMode.UNDERSCORE, '*': OptionNameMode.BOTH}
+
 
 # endregion
 
 
 class ConfigItem(Generic[_ConfigValue]):
+    __slots__ = ('default', 'type', 'name')
+
     def __init__(self, default: ConfigValue, type: Callable[[Any], _ConfigValue] = None):  # noqa
         self.default = default
         self.type = type
@@ -230,7 +248,7 @@ class CommandConfig:
             raise ValueError(f'Invalid configuration - unsupported options: {", ".join(sorted(bad))}')
 
     def __repr__(self) -> str:
-        settings = ', '.join(f'{k}={v!r}' for k, v in self.as_dict(False))
+        settings = ', '.join(f'{k}={v!r}' for k, v in self.as_dict(False).items())
         cfg_str = f', {settings}' if settings else ''
         return f'<{self.__class__.__name__}(parents={self.parents!r}{cfg_str})>'
 
