@@ -128,7 +128,7 @@ class ParamBase(ABC):
     def __hash__(self) -> int:
         return reduce(xor, map(hash, (self.__class__, self.__name, self.name, self.command)))
 
-    def _ctx(self, command: CommandType = None) -> Context:
+    def _ctx(self, command: Command = None) -> Context:
         try:
             return get_current_context()
         except NoActiveContext:
@@ -141,7 +141,7 @@ class ParamBase(ABC):
             pass
         raise NoActiveContext('There is no active context')
 
-    def _config(self, command: CommandType = None) -> CommandConfig:
+    def _config(self, command: Command = None) -> CommandConfig:
         if command is None:
             command = self.command
         try:
@@ -320,7 +320,9 @@ class Parameter(ParamBase, ABC):
         except TypeError:
             return False
 
-    def take_action(self, value: Optional[str], short_combo: bool = False):
+    def take_action(  # pylint: disable=W0613
+        self, value: Optional[str], short_combo: bool = False, opt_str: str = None
+    ):
         # log.debug(f'{self!r}.take_action({value!r})')
         action = self.action
         if action == 'append' and self._nargs_max_reached():
@@ -334,17 +336,10 @@ class Parameter(ParamBase, ABC):
                 raise ParamUsageError(self, f'received value={value!r} but a stored value={val!r} already exists')
 
         ctx.record_action(self)
-        action_method = getattr(self, self.action)
-        if action in {'store_const', 'append_const'}:
-            if value is not None:
-                raise ParamUsageError(
-                    self, f'received value={value!r} but no values are accepted for action={action!r}'
-                )
-            return action_method()
-        else:
-            normalized = self.prepare_value(value, short_combo) if value is not None else value
-            self.validate(normalized)
-            return action_method(normalized)
+        action_method = getattr(self, action)
+        normalized = self.prepare_value(value, short_combo) if value is not None else value
+        self.validate(normalized)
+        return action_method(normalized)
 
     def would_accept(self, value: str, short_combo: bool = False) -> bool:
         action = self.action
@@ -510,18 +505,28 @@ class BasePositional(Parameter, ABC):
     """
 
     _positional: bool = True
+    _default_ok: bool = False
 
-    def __init__(self, action: str, **kwargs):
-        required = kwargs.setdefault('required', True)
-        if not required:
+    def __init_subclass__(cls, default_ok: bool = None, **kwargs):  # pylint: disable=W0222
+        """
+        :param default_ok: Whether default values are supported for this Parameter type
+        :param kwargs: Additional keyword arguments to pass to :meth:`.Parameter.__init_subclass__`.
+        """
+        super().__init_subclass__(**kwargs)
+        if default_ok is not None:
+            cls._default_ok = default_ok
+
+    def __init__(self, action: str, *, required: Bool = True, default: Any = _NotSet, **kwargs):
+        default_bad = not self._default_ok or 0 not in self.nargs
+        if not required and default_bad:
             cls_name = self.__class__.__name__
             raise ParameterDefinitionError(
                 f'All {cls_name} parameters must be required - invalid required={required!r}'
             )
-        elif kwargs.setdefault('default', _NotSet) is not _NotSet:
+        elif default_bad and default is not _NotSet:
             cls_name = self.__class__.__name__
             raise ParameterDefinitionError(f"The 'default' arg is not supported for {cls_name} parameters")
-        super().__init__(action, **kwargs)
+        super().__init__(action, default=default, required=required, **kwargs)
 
 
 class BaseOption(Parameter, ABC):
@@ -549,16 +554,7 @@ class BaseOption(Parameter, ABC):
     name_mode: Optional[OptionNameMode] = None  # OptionNameMode override
 
     def __init__(self, *option_strs: str, action: str, name_mode: Union[OptionNameMode, str] = None, **kwargs):
-        bad_opts = ', '.join(opt for opt in option_strs if not 0 < opt.count('-', 0, 3) < 3)
-        if bad_opts:
-            raise ParameterDefinitionError(f"Bad option(s) - must start with '--' or '-': {bad_opts}")
-        bad_opts = ', '.join(opt for opt in option_strs if opt.endswith('-'))
-        if bad_opts:
-            raise ParameterDefinitionError(f"Bad option(s) - may not end with '-': {bad_opts}")
-        bad_opts = ', '.join(opt for opt in option_strs if '=' in opt)
-        if bad_opts:
-            raise ParameterDefinitionError(f"Bad option(s) - may not contain '=': {bad_opts}")
-
+        _validate_opt_strs(option_strs)
         super().__init__(action, **kwargs)
         self._long_opts = {opt for opt in option_strs if opt.startswith('--')}
         self._short_opts = short_opts = {opt for opt in option_strs if 1 == opt.count('-', 0, 2)}
@@ -591,6 +587,13 @@ class BaseOption(Parameter, ABC):
         return sorted(self._short_opts, key=lambda opt: (-len(opt), opt))
 
 
+def _validate_opt_strs(opt_strs: Collection[str]):
+    bad = ', '.join(opt for opt in opt_strs if not 0 < opt.count('-', 0, 3) < 3 or opt.endswith('-') or '=' in opt)
+    if bad:
+        msg = f"Bad option(s) - they must start with '--' or '-', may not end with '-', and may not contain '=': {bad}"
+        raise ParameterDefinitionError(msg)
+
+
 def _is_numeric(text: str) -> Bool:
     try:
         num_match = _is_numeric._num_match
